// Copyright (C) 2019 ~ 2020 Uniontech Software Technology Co.,Ltd
// SPDX-FileCopyrightText: 2022 UnionTech Software Technology Co., Ltd.
//
// SPDX-License-Identifier: GPL-3.0-or-later

// This file was generated by dtk-settings-tools version 0.1.2

#include <DSettings>
#include <QDebug>

/*******************************************************************************
 1. @函数:    GenerateSettingTranslate
 2. @作者:    ut000610 daizhengwen
 3. @日期:    2020-08-11
 4. @说明:    生成设置翻译
*******************************************************************************/
void GenerateSettingTranslate()
{
    qDebug() << "GenerateSettingTranslate start";
    auto advanced_cursor_auto_copy_selectionText = QObject::tr("Copy on select");
    Q_UNUSED(advanced_cursor_auto_copy_selectionText);
    auto advanced_cursor_cursor_blinkText = QObject::tr("Cursor blink");
    Q_UNUSED(advanced_cursor_cursor_blinkText);
    auto advanced_cursor_cursor_shapeName = QObject::tr("Cursor style");
    Q_UNUSED(advanced_cursor_cursor_shapeName);
    auto advanced_scroll_scroll_on_keyText = QObject::tr("Scroll on keystroke");
    Q_UNUSED(advanced_scroll_scroll_on_keyText);
    auto advanced_scroll_scroll_on_outputText = QObject::tr("Scroll on output");
    Q_UNUSED(advanced_scroll_scroll_on_outputText);
    auto advanced_scroll_zoom_on_ctrl_scrollwheel = QObject::tr("Allow Ctrl+scrollwheel to zoom text size");
    Q_UNUSED(advanced_scroll_zoom_on_ctrl_scrollwheel);
    auto advanced_scroll_opacity_on_ctrl_shift_updown = QObject::tr("Allow Ctrl+Shift+Up/Down to adjust opacity");
    Q_UNUSED(advanced_scroll_opacity_on_ctrl_shift_updown);
    auto advanced_window_auto_hide_raytheon_windowText = QObject::tr("Hide Quake window after losing focus");
    Q_UNUSED(advanced_window_auto_hide_raytheon_windowText);
    auto advanced_window_quake_window_durationText = QObject::tr("Quake window animation speed");
    Q_UNUSED(advanced_window_quake_window_durationText);
    auto basic_interface_blurred_backgroundText = QObject::tr("Blur background");
    Q_UNUSED(basic_interface_blurred_backgroundText);
    auto advanced_window_use_on_startingName = QObject::tr("Use on starting");
    Q_UNUSED(advanced_window_use_on_startingName);
    auto basic_interface_fontName = QObject::tr("Font");
    Q_UNUSED(basic_interface_fontName);
    auto basic_interface_font_sizeName = QObject::tr("Font size");
    Q_UNUSED(basic_interface_font_sizeName);
    auto basic_interface_opacityName = QObject::tr("Opacity");
    Q_UNUSED(basic_interface_opacityName);
    auto group_advancedName = QObject::tr("Advanced");
    Q_UNUSED(group_advancedName);
    auto group_advanced_cursorName = QObject::tr("Cursor");
    Q_UNUSED(group_advanced_cursorName);
    auto group_advanced_scrollName = QObject::tr("Scroll");
    Q_UNUSED(group_advanced_scrollName);
    auto group_advanced_windowName = QObject::tr("Window");
    Q_UNUSED(group_advanced_windowName);
    auto group_basicName = QObject::tr("Basic");
    Q_UNUSED(group_basicName);
    auto group_basic_interfaceName = QObject::tr("Interface");
    Q_UNUSED(group_basic_interfaceName);
    auto group_shortcutsName = QObject::tr("Shortcuts");
    Q_UNUSED(group_shortcutsName);
    auto group_shortcuts_advancedName = QObject::tr("Others");
    Q_UNUSED(group_shortcuts_advancedName);
    auto group_shortcuts_terminalName = QObject::tr("Terminal");
    Q_UNUSED(group_shortcuts_terminalName);
    auto group_shortcuts_workspaceName = QObject::tr("Workspace");
    Q_UNUSED(group_shortcuts_workspaceName);
    auto shortcuts_advanced_custom_commandName = QObject::tr("Custom commands");
    Q_UNUSED(shortcuts_advanced_custom_commandName);
    auto shortcuts_advanced_display_shortcutsName = QObject::tr("Display shortcuts");
    Q_UNUSED(shortcuts_advanced_display_shortcutsName);
    auto shortcuts_advanced_remote_managementName = QObject::tr("Remote management");
    Q_UNUSED(shortcuts_advanced_remote_managementName);
    auto shortcuts_advanced_rename_tabName = QObject::tr("Rename title");
    Q_UNUSED(shortcuts_advanced_rename_tabName);
    auto shortcuts_advanced_switch_fullscreenName = QObject::tr("Fullscreen");
    Q_UNUSED(shortcuts_advanced_switch_fullscreenName);
    auto shortcuts_terminal_copyName = QObject::tr("Copy");
    Q_UNUSED(shortcuts_terminal_copyName);
    auto shortcuts_terminal_default_sizeName = QObject::tr("Default size");
    Q_UNUSED(shortcuts_terminal_default_sizeName);
    auto shortcuts_terminal_pasteName = QObject::tr("Paste");
    Q_UNUSED(shortcuts_terminal_pasteName);
    auto shortcuts_terminal_searchName = QObject::tr("Search");
    Q_UNUSED(shortcuts_terminal_searchName);
    auto shortcuts_terminal_select_allName = QObject::tr("Select all");
    Q_UNUSED(shortcuts_terminal_select_allName);
    auto shortcuts_terminal_skip_to_next_commandName = QObject::tr("Jump to next command");
    Q_UNUSED(shortcuts_terminal_skip_to_next_commandName);
    auto shortcuts_terminal_skip_to_previous_commandName = QObject::tr("Jump to previous command");
    Q_UNUSED(shortcuts_terminal_skip_to_previous_commandName);
    auto shortcuts_terminal_zoom_inName = QObject::tr("Zoom in");
    Q_UNUSED(shortcuts_terminal_zoom_inName);
    auto shortcuts_terminal_zoom_outName = QObject::tr("Zoom out");
    Q_UNUSED(shortcuts_terminal_zoom_outName);
    auto shortcuts_workspace_close_other_windowsName = QObject::tr("Close other windows");
    Q_UNUSED(shortcuts_workspace_close_other_windowsName);
    auto shortcuts_workspace_close_other_workspace = QObject::tr("Close other workspaces");
    Q_UNUSED(shortcuts_workspace_close_other_workspace);
    auto shortcuts_workspace_close_windowName = QObject::tr("Close window");
    Q_UNUSED(shortcuts_workspace_close_windowName);
    auto shortcuts_workspace_close_workspaceName = QObject::tr("Close workspace");
    Q_UNUSED(shortcuts_workspace_close_workspaceName);
    auto shortcuts_workspace_horionzal_splitName = QObject::tr("Horizontal split");
    Q_UNUSED(shortcuts_workspace_horionzal_splitName);
    auto shortcuts_tab_new_tabName = QObject::tr("New tab");
    Q_UNUSED(shortcuts_tab_new_tabName);
    auto shortcuts_workspace_next_tabName = QObject::tr("Next tab");
    Q_UNUSED(shortcuts_workspace_next_tabName);
    auto shortcuts_workspace_previous_tabName = QObject::tr("Previous tab");
    Q_UNUSED(shortcuts_workspace_previous_tabName);
    auto shortcuts_workspace_select_left_workspaceName = QObject::tr("Select left workspace");
    Q_UNUSED(shortcuts_workspace_select_left_workspaceName);
    auto shortcuts_workspace_select_lower_workspaceName = QObject::tr("Select lower workspace");
    Q_UNUSED(shortcuts_workspace_select_lower_workspaceName);
    auto shortcuts_workspace_select_right_workspaceName = QObject::tr("Select right workspace");
    Q_UNUSED(shortcuts_workspace_select_right_workspaceName);
    auto shortcuts_workspace_select_upper_workspaceName = QObject::tr("Select upper workspace");
    Q_UNUSED(shortcuts_workspace_select_upper_workspaceName);
    auto shortcuts_workspace_vertical_splitName = QObject::tr("Vertical split");
    Q_UNUSED(shortcuts_workspace_vertical_splitName);
    auto shortcuts_terminal_FindName = QObject::tr("Find");
    Q_UNUSED(shortcuts_terminal_FindName);
    auto tab_titles = QObject::tr("Tab titles");
    Q_UNUSED(tab_titles);
    auto tab_title_format = QObject::tr("Tab title format");
    Q_UNUSED(tab_title_format);
    auto remote_tab_title_format = QObject::tr("Remote tab title format");
    Q_UNUSED(remote_tab_title_format);
    auto go_to_tab_1 = QObject::tr("Go to tab 1");
    Q_UNUSED(go_to_tab_1);
    auto go_to_tab_2 = QObject::tr("Go to tab 2");
    Q_UNUSED(go_to_tab_2);
    auto go_to_tab_3 = QObject::tr("Go to tab 3");
    Q_UNUSED(go_to_tab_3);
    auto go_to_tab_4 = QObject::tr("Go to tab 4");
    Q_UNUSED(go_to_tab_4);
    auto go_to_tab_5 = QObject::tr("Go to tab 5");
    Q_UNUSED(go_to_tab_5);
    auto go_to_tab_6 = QObject::tr("Go to tab 6");
    Q_UNUSED(go_to_tab_6);
    auto go_to_tab_7 = QObject::tr("Go to tab 7");
    Q_UNUSED(go_to_tab_7);
    auto go_to_tab_8 = QObject::tr("Go to tab 8");
    Q_UNUSED(go_to_tab_8);
    auto go_to_tab_9 = QObject::tr("Go to tab 9");
    Q_UNUSED(go_to_tab_9);
    auto disable_flow_control_using_ctrl_s_ctrl_q = QObject::tr("Disable flow control using Ctrl+S, Ctrl+Q");
    Q_UNUSED(disable_flow_control_using_ctrl_s_ctrl_q);
    auto shell_profile = QObject::tr("Shell profile");
    Q_UNUSED(shell_profile);

    qDebug() << "GenerateSettingTranslate end";
    auto history_size = QObject::tr("History size");
    Q_UNUSED(history_size);
    auto include_special_characters_in_double_click_selectionisText = QObject::tr("Include special character(s) in double click selections");
    Q_UNUSED(include_special_characters_in_double_click_selectionisText);
    auto set_cursor_position = QObject::tr("Allow Ctrl + left mouse click to set cursor position");
    Q_UNUSED(set_cursor_position);
    auto enable_debuginfod = QObject::tr("The HTTP file server used for transmitting debugging information resources");
    Q_UNUSED(enable_debuginfod);

    // OSC52 clipboard
    auto advanced_cursor_allow_osc52Text = QObject::tr("Allow OSC52 clipboard access");
    Q_UNUSED(advanced_cursor_allow_osc52Text);
}
